package com.feiyizhan;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

/**
 * Log Jcl 的测试 (Jcl 是commons-logging 日志框架）
 * jcl支持log4j,jul,jdk13等日志，它是通过动态Class.forName加载的，
 * 优先级为log4j->jul(>=jdk14自带)->jdk13->simpleLog顺序打印,比如动态Class.forName log4j包没有时候会自动再加载jul包，以此顺序，
 * 具体可以看log4j包中org.apache.commons.logging.impl.LogFactoryImpl类中classesToDiscover数组中加载顺序。
 * 不支持主流的log4j2,logback等,因此现在基本不用，或者用也会桥接到slf4j上。
 * @author 徐明龙 XuMingLong 2023-10-18
 */
@SpringBootTest
public class LogJclTests {

    Log logger = LogFactory.getLog(LogJclTests.class.getName());


    @Test
    void test_default_log(){
        logger.trace("这是trace日志..."); //跟踪轨迹
        logger.debug("这是debug日志..."); //调试
        //SpringBoot默认给我们使用的是info级别的，没有指定级别的就用SpringBoot默认规定的级别（root级别）
        logger.info("这是info日志...");
        logger.warn("这是warn日志...");
        logger.error("这是error日志...");
        logger.info(logger.getClass().toGenericString());
    }
}
