# Java Log SLF4j Jcl binding demo
测试`Slf4j`  binding （桥接到 `Jcl` ）的场景
将原本的`Logback`、`Log4j2`、`Jul`、`Jcl`、`log4j`的输出，统一到`Sfl4j` 并最终使用`Jcl`输出日志
要实现这个效果就必须引入对应日志框架的`Slf4j`的Over包，将日志重定向到`Slf4j`。但由于`Slf4j `并没有一个具体的日志实现，所以还得找一个具体的日志实现，这个demo选择的是`Jcl`作为最终的日志实现。
而`Jcl`本身也是一个日志门面，需要配合具体的实现来做日志输出，这个demo使用`Log4j`作为`Jcl`的日志实现。

## 引入对应Over包
```xml
<!-- jcl over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jcl-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- lo4j over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>log4j-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- jul over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jul-to-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- LOG4J2 over SLF4j -->
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-to-slf4j</artifactId>
    <version>${log4j2.version}</version>
</dependency>
```


## `Logback` Over 处理
`Logback` 本身就支持并使用了`slf4j`门面 ,没有要`Over`也不需要`Over`，因此不参与本测试。

## `Jcl` Over 处理
`Jcl` 由于Spring 自己做了一个Over的包`spring-jcl`，因此如果需要测试验证，还要把这个包给排除掉。
本demo最终使用`Jcl`作为输出，因此排除`Jcl`的`Over`包
```xml
<!-- jcl over slf4j -->
<!--<dependency>-->
<!--  <groupId>org.slf4j</groupId>-->
<!--  <artifactId>jcl-over-slf4j</artifactId>-->
<!--  <version>${slf4j.version}</version>-->
<!--</dependency>-->
```

## `Log4j` Over 处理
本demo 最终使用`Log4j`做`Jcl`的实现，不能再对`Log4j` 做`Over`,因此需要排除`log4j-over-slf4j` 包。
```xml
<!-- lo4j over slf4j -->
<!--<dependency>-->
<!--  <groupId>org.slf4j</groupId>-->
<!--  <artifactId>log4j-over-slf4j</artifactId>-->
<!--  <version>${slf4j.version}</version>-->
<!--</dependency>-->
```

## `Log4j2` Over 处理
`log4j2` 引入`log4j-to-slf4j` 这个包。

还需要排除`log4j-core` 包，当然如果同时存在`logback-classic` 则不排除也可以正常运行，否则会抛出异常`org.apache.logging.slf4j.SLF4JLoggerContext cannot be cast to org.apache.logging.log4j.core.LoggerContext`
。因为Spring 默认会优先加载Logback的`LoggingSystem`的一个实现`LogbackLoggingSystem`，当没有的时候才会选择`Log4J2LoggingSystem`，但当没有`Logback`实现,并且存在`log4j-core`时，
将会使用`Log4J2LoggingSystem`作为日志的实现，其实也就是一个动态的加载日志实现系统的处理。但`log4j-core`不能直接转换到`Slf4j`,因此才会抛出上面的异常。

```xml
<!--		<dependency>-->
<!--			<groupId>org.apache.logging.log4j</groupId>-->
<!--			<artifactId>log4j-core</artifactId>-->
<!--			<version>${log4j2.version}</version>-->
<!--		</dependency>-->
```

## `Jul` Over 处理
引入`jul-to-slf4j` 包。

Jul  在非Spring Boot环境下默认日志配置文件: `JVM/lib/logging.properties`
Spring Boot中 Jul 的默认配置文件 `spring-boot-2.7.9.jar!\org\springframework\boot\logging\java\logging.properties`
可以通过 在`application.properties` 增加下面的配置来指定自定义的配置文件
`logging.config=classpath:logging.properties`

参考https://www.slf4j.org/legacy.html，https://www.slf4j.org/api/org/slf4j/bridge/SLF4JBridgeHandler.html  文档，修改默认Handlers，否则Over是不生效的。
简单的方法就是在`logging.properties` 替换`handlers =java.util.logging.ConsoleHandler`配置为`handlers = org.slf4j.bridge.SLF4JBridgeHandler`


> 注意`jul-to-slf4j` 和`slf4j-jdk14` 不能同时引入，因为一个是将`Jul`重定向到`slf4j`，另一个是将`slf4j`的日志输出使用`Jul`来输出。



## 引入 `SLF4J` binding 到 `jcl` 的包
```xml
<!-- Jcl binding slf4j -->
<dependency>
  <groupId>org.slf4j</groupId>
  <artifactId>slf4j-jcl</artifactId>
  <version>${slf4j.version}</version>
  <exclusions>
    <exclusion>
      <artifactId>commons-logging</artifactId>
      <groupId>commons-logging</groupId>
    </exclusion>
  </exclusions>
</dependency>
```


## 没有引入对应over依赖包的情况下的输出
日志各自使用各自的输出。
```log
2023-11-23 15:15:59 DEBUG (com.feiyizhan.LogJclTests:21) - LogJclTests 这是debug日志...
2023-11-23 15:15:59  INFO (com.feiyizhan.LogJclTests:23) - LogJclTests 这是info日志...
2023-11-23 15:15:59  WARN (com.feiyizhan.LogJclTests:24) - LogJclTests 这是warn日志...
2023-11-23 15:15:59 ERROR (com.feiyizhan.LogJclTests:25) - LogJclTests 这是error日志...
2023-11-23 15:15:59  INFO (com.feiyizhan.LogJclTests:26) - LogJclTests public class org.apache.commons.logging.impl.Log4JLogger
十一月 23, 2023 3:15:59 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests 这是info日志...
十一月 23, 2023 3:15:59 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests public class java.util.logging.Logger
2023-11-23 15:15:59 DEBUG (com.feiyizhan.LogLog4jTests:21) - LogLog4jTests 这是debug日志...
2023-11-23 15:15:59  INFO (com.feiyizhan.LogLog4jTests:23) - LogLog4jTests 这是info日志...
2023-11-23 15:15:59  WARN (com.feiyizhan.LogLog4jTests:24) - LogLog4jTests 这是warn日志...
2023-11-23 15:15:59 ERROR (com.feiyizhan.LogLog4jTests:25) - LogLog4jTests 这是error日志...
2023-11-23 15:15:59  INFO (com.feiyizhan.LogLog4jTests:26) - LogLog4jTests public class org.apache.log4j.Logger
15:15:59.124 [main] ERROR com.feiyizhan.LogLog4j2Tests - 这是error日志...

```

## 引入对应over依赖包的情况下的输出
全部统一一种日志输出格式了。
```log
2023-11-23 15:20:24 DEBUG (com.feiyizhan.LogJclTests:21) - LogJclTests 这是debug日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogJclTests:23) - LogJclTests 这是info日志...
2023-11-23 15:20:24  WARN (com.feiyizhan.LogJclTests:24) - LogJclTests 这是warn日志...
2023-11-23 15:20:24 ERROR (com.feiyizhan.LogJclTests:25) - LogJclTests 这是error日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogJclTests:26) - LogJclTests public class org.apache.commons.logging.impl.Log4JLogger
2023-11-23 15:20:24  INFO (com.feiyizhan.LogJulTests:338) - LogJulTests 这是info日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogJulTests:338) - LogJulTests public class java.util.logging.Logger
2023-11-23 15:20:24 DEBUG (com.feiyizhan.LogLog4jTests:21) - LogLog4jTests 这是debug日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogLog4jTests:23) - LogLog4jTests 这是info日志...
2023-11-23 15:20:24  WARN (com.feiyizhan.LogLog4jTests:24) - LogLog4jTests 这是warn日志...
2023-11-23 15:20:24 ERROR (com.feiyizhan.LogLog4jTests:25) - LogLog4jTests 这是error日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogLog4jTests:26) - LogLog4jTests public class org.apache.log4j.Logger
2023-11-23 15:20:24 DEBUG (com.feiyizhan.LogLog4j2Tests:242) - 这是debug日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogLog4j2Tests:338) - 这是info日志...
2023-11-23 15:20:24  WARN (com.feiyizhan.LogLog4j2Tests:433) - 这是warn日志...
2023-11-23 15:20:24 ERROR (com.feiyizhan.LogLog4j2Tests:528) - 这是error日志...
2023-11-23 15:20:24  INFO (com.feiyizhan.LogLog4j2Tests:338) - 这是info日志...test
2023-11-23 15:20:24  INFO (com.feiyizhan.LogLog4j2Tests:338) - public class org.apache.logging.slf4j.SLF4JLogger

```


## 版本冲突的问题
`slf4j-jcl` 最高只支持`slf4j` `1.7.x` 版本，因此必须要将slf4j 降到 1.7.x版本才能使用

异常信息如下：
```log
SLF4J: No SLF4J providers were found.
SLF4J: Defaulting to no-operation (NOP) logger implementation
SLF4J: See https://www.slf4j.org/codes.html#noProviders for further details.
SLF4J: Class path contains SLF4J bindings targeting slf4j-api versions 1.7.x or earlier.
SLF4J: Ignoring binding found at [jar:file:/D:/repository/org/slf4j/slf4j-jcl/1.7.36/slf4j-jcl-1.7.36.jar!/org/slf4j/impl/StaticLoggerBinder.class]
SLF4J: See https://www.slf4j.org/codes.html#ignoredBindings for an explanation.
```