# Java Log SLF4j Log4j2 binding demo
测试Slf4j binding （桥接到 `Log4j2` ）的场景
将原本的`Logback`、`Log4j2`、`Jul`、`Jcl`、`log4j`的输出，统一到`Sfl4j` 并最终使用`Log4j2`输出日志
要实现这个效果就必须引入对应日志框架的`Slf4j`的Over包，将日志重定向到`Slf4j`。但由于`Slf4j `并没有一个具体的日志实现，所以还得找一个具体的日志实现，这个demo选择的是`Log4j2`作为最终的日志实现。

## 引入对应Over包
```xml
<!-- jcl over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jcl-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- lo4j over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>log4j-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- jul over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jul-to-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- LOG4J2 over SLF4j -->
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-to-slf4j</artifactId>
    <version>${log4j2.version}</version>
</dependency>
```

## `Logback` Over 处理
`Logback` 本身就支持并使用了`slf4j`门面 ,没有要`Over`也不需要`Over`，因此不参与本测试。

## `Jcl` Over 处理
`Jcl` 由于Spring 自己做了一个Over的包`spring-jcl`，因此如果需要测试验证，还要把这个包给排除掉。
然后引入`jcl-over-slf4j` 包就可以，当然也可以用`spring-jcl`，效果一样。

## `Log4j` Over 处理
`Log4j` 除了需要引入`log4j-over-slf4j` 包外，还要排除掉`log4j` 本身的依赖包，否则Over是无效的。
```xml
<!--引入Log4j-->
<!--		<dependency>-->
<!--			<groupId>log4j</groupId>-->
<!--			<artifactId>log4j</artifactId>-->
<!--			<version>1.2.17</version>-->
<!--		</dependency>-->
```

## `Log4j2` Over 处理
本Demo是最终是使用`log4j2` 输出，因此不能再把`Log4j2` Over掉了，因此不做Over处理。
```xml
<!-- LOG4J2 over SLF4j -->
<!--<dependency>-->
<!--    <groupId>org.apache.logging.log4j</groupId>-->
<!--    <artifactId>log4j-to-slf4j</artifactId>-->
<!--    <version>${log4j2.version}</version>-->
<!--</dependency>-->
```

## `Jul` Over 处理
引入`jul-to-slf4j` 包。

Jul  在非Spring Boot环境下默认日志配置文件: `JVM/lib/logging.properties`
Spring Boot中 Jul 的默认配置文件 `spring-boot-2.7.9.jar!\org\springframework\boot\logging\java\logging.properties`
可以通过 在`application.properties` 增加下面的配置来指定自定义的配置文件
`logging.config=classpath:logging.properties`

参考https://www.slf4j.org/legacy.html，https://www.slf4j.org/api/org/slf4j/bridge/SLF4JBridgeHandler.html  文档，修改默认Handlers，否则Over是不生效的。
简单的方法就是在`logging.properties` 替换`handlers =java.util.logging.ConsoleHandler`配置为`handlers = org.slf4j.bridge.SLF4JBridgeHandler`

## `Slf4j` Binding `Log4j2`  的处理

引入`slf4j` 的`Log4j2`实现包`log4j-slf4j-impl`
```xml
<!-- slf4j binding Log4j2 -->
<dependency>
  <groupId>org.apache.logging.log4j</groupId>
  <artifactId>log4j-slf4j-impl</artifactId>
  <version>${log4j2.version}</version>
</dependency>
```


## 没有引入对应over依赖包的情况下的输出
日志各自使用各自的输出，`JCL` 在有`Log4j`的时候，使用的就是`Log4j`实现。
```log
2023-10-27 14:26:29 DEBUG (com.feiyizhan.LogJclTests:21) - LogJclTests 这是debug日志...
2023-10-27 14:26:29  INFO (com.feiyizhan.LogJclTests:23) - LogJclTests 这是info日志...
2023-10-27 14:26:29  WARN (com.feiyizhan.LogJclTests:24) - LogJclTests 这是warn日志...
2023-10-27 14:26:29 ERROR (com.feiyizhan.LogJclTests:25) - LogJclTests 这是error日志...
2023-10-27 14:26:29  INFO (com.feiyizhan.LogJclTests:26) - LogJclTests public class org.apache.commons.logging.impl.Log4JLogger
十月 27, 2023 2:26:29 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests 这是info日志...
十月 27, 2023 2:26:29 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests public class java.util.logging.Logger
2023-10-27 14:26:29 DEBUG (com.feiyizhan.LogLog4jTests:21) - LogLog4jTests 这是debug日志...
2023-10-27 14:26:29  INFO (com.feiyizhan.LogLog4jTests:23) - LogLog4jTests 这是info日志...
2023-10-27 14:26:29  WARN (com.feiyizhan.LogLog4jTests:24) - LogLog4jTests 这是warn日志...
2023-10-27 14:26:29 ERROR (com.feiyizhan.LogLog4jTests:25) - LogLog4jTests 这是error日志...
2023-10-27 14:26:29  INFO (com.feiyizhan.LogLog4jTests:26) - LogLog4jTests public class org.apache.log4j.Logger
[2023-10-27 14:26:29.360] [] [main] INFO : [c.f.LogLog4j2Tests:19] - 这是info日志...
[2023-10-27 14:26:29.363] [] [main] WARN : [c.f.LogLog4j2Tests:20] - 这是warn日志...
[2023-10-27 14:26:29.363] [] [main] ERROR: [c.f.LogLog4j2Tests:21] - 这是error日志...
[2023-10-27 14:26:29.377] [] [main] INFO : [c.f.LogLog4j2Tests:22] - 这是info日志...test
[2023-10-27 14:26:29.377] [] [main] INFO : [c.f.LogLog4j2Tests:23] - public class org.apache.logging.slf4j.Log4jLogger
```

## 引入对应over依赖包的情况下的输出
全部统一一种日志输出格式了。
```log
[2023-10-27 14:25:05.314] [] [main] INFO : [c.f.LogJclTests:23] - LogJclTests 这是info日志...
[2023-10-27 14:25:05.314] [] [main] WARN : [c.f.LogJclTests:24] - LogJclTests 这是warn日志...
[2023-10-27 14:25:05.314] [] [main] ERROR: [c.f.LogJclTests:25] - LogJclTests 这是error日志...
[2023-10-27 14:25:05.314] [] [main] INFO : [c.f.LogJclTests:26] - LogJclTests public class org.apache.commons.logging.impl.SLF4JLocationAwareLog
[2023-10-27 14:25:05.316] [] [main] INFO : [c.f.LogJulTests:19] - LogJulTests 这是info日志...
[2023-10-27 14:25:05.316] [] [main] INFO : [c.f.LogJulTests:20] - LogJulTests public class java.util.logging.Logger
[2023-10-27 14:25:05.319] [] [main] INFO : [c.f.LogLog4jTests:23] - LogLog4jTests 这是info日志...
[2023-10-27 14:25:05.320] [] [main] WARN : [c.f.LogLog4jTests:24] - LogLog4jTests 这是warn日志...
[2023-10-27 14:25:05.320] [] [main] ERROR: [c.f.LogLog4jTests:25] - LogLog4jTests 这是error日志...
[2023-10-27 14:25:05.320] [] [main] INFO : [c.f.LogLog4jTests:26] - LogLog4jTests public class org.apache.log4j.Logger
[2023-10-27 14:25:05.322] [] [main] INFO : [c.f.LogLog4j2Tests:19] - 这是info日志...
[2023-10-27 14:25:05.322] [] [main] WARN : [c.f.LogLog4j2Tests:20] - 这是warn日志...
[2023-10-27 14:25:05.323] [] [main] ERROR: [c.f.LogLog4j2Tests:21] - 这是error日志...
[2023-10-27 14:25:05.329] [] [main] INFO : [c.f.LogLog4j2Tests:22] - 这是info日志...test
[2023-10-27 14:25:05.329] [] [main] INFO : [c.f.LogLog4j2Tests:23] - public class org.apache.logging.slf4j.Log4jLogger
```
