# Java Log SLF4j Logback binding demo
测试Slf4j binding （桥接到 `Logback` ）的场景
将原本的`Logback`、`Log4j2`、`Jul`、`Jcl`、`log4j`的输出，统一到`Sfl4j` 并最终使用`Logback`输出日志
要实现这个效果就必须引入对应日志框架的`Slf4j`的Over包，将日志重定向到`Slf4j`。但由于`Slf4j `并没有一个具体的日志实现，所以还得找一个具体的日志实现，这个demo选择的是`Logback`作为最终的日志实现。

## 引入对应Over包
```xml
<!-- jcl over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jcl-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- lo4j over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>log4j-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- jul over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jul-to-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- LOG4J2 over SLF4j -->
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-to-slf4j</artifactId>
    <version>${log4j2.version}</version>
</dependency>
```

## `Logback` Over 处理
本Demo是最终是使用`Logback` 输出，因此不能再把`Logback` Over掉了，因此不做Over处理。

## `Jcl` Over 处理
`Jcl` 由于Spring 自己做了一个Over的包`spring-jcl`，因此如果需要测试验证，还要把这个包给排除掉。
然后引入`jcl-over-slf4j` 包就可以，当然也可以用`spring-jcl`，效果一样。

## `Log4j` Over 处理
`Log4j` 除了需要引入`log4j-over-slf4j` 包外，还要排除掉`log4j` 本身的依赖包，否则Over是无效的。
```xml
<!--引入Log4j-->
<!--		<dependency>-->
<!--			<groupId>log4j</groupId>-->
<!--			<artifactId>log4j</artifactId>-->
<!--			<version>1.2.17</version>-->
<!--		</dependency>-->
```

## `Log4j2` Over 处理
`log4j2` 引入`log4j-to-slf4j` 这个包。

还需要排除`log4j-core` 包，当然如果同时存在`logback-classic` 则不排除也可以正常运行，否则会抛出异常`org.apache.logging.slf4j.SLF4JLoggerContext cannot be cast to org.apache.logging.log4j.core.LoggerContext`
。因为Spring 默认会优先加载Logback的`LoggingSystem`的一个实现`LogbackLoggingSystem`，当没有的时候才会选择`Log4J2LoggingSystem`，但当没有`Logback`实现,并且存在`log4j-core`时，
将会使用`Log4J2LoggingSystem`作为日志的实现，其实也就是一个动态的加载日志实现系统的处理。但`log4j-core`不能直接转换到`Slf4j`,因此才会抛出上面的异常。

```xml
<!--		<dependency>-->
<!--			<groupId>org.apache.logging.log4j</groupId>-->
<!--			<artifactId>log4j-core</artifactId>-->
<!--			<version>${log4j2.version}</version>-->
<!--		</dependency>-->
```

## `Jul` Over 处理
引入`jul-to-slf4j` 包。

Jul  在非Spring Boot环境下默认日志配置文件: `JVM/lib/logging.properties`
Spring Boot中 Jul 的默认配置文件 `spring-boot-2.7.9.jar!\org\springframework\boot\logging\java\logging.properties`
可以通过 在`application.properties` 增加下面的配置来指定自定义的配置文件
`logging.config=classpath:logging.properties`

参考https://www.slf4j.org/legacy.html，https://www.slf4j.org/api/org/slf4j/bridge/SLF4JBridgeHandler.html  文档，修改默认Handlers，否则Over是不生效的。
简单的方法就是在`logging.properties` 替换`handlers =java.util.logging.ConsoleHandler`配置为`handlers = org.slf4j.bridge.SLF4JBridgeHandler`



## `Slf4j` Binding `Logback` 的处理

引入`slf4j` 的`Logback`实现包`logback-classic`
```xml
<!-- SLF4j binding logback-->
<dependency>
  <groupId>ch.qos.logback</groupId>
  <artifactId>logback-classic</artifactId>
  <version>1.2.11</version>
</dependency>
```


## 没有引入对应over依赖包的情况下的输出
日志各自使用各自的输出，`JCL` 在有`Log4j`的时候，使用的就是`Log4j`实现。
```log
2023-11-03 17:19:24 DEBUG (com.feiyizhan.LogJclTests:21) - LogJclTests 这是debug日志...
2023-11-03 17:19:24  INFO (com.feiyizhan.LogJclTests:23) - LogJclTests 这是info日志...
2023-11-03 17:19:24  WARN (com.feiyizhan.LogJclTests:24) - LogJclTests 这是warn日志...
2023-11-03 17:19:24 ERROR (com.feiyizhan.LogJclTests:25) - LogJclTests 这是error日志...
2023-11-03 17:19:24  INFO (com.feiyizhan.LogJclTests:26) - LogJclTests public class org.apache.commons.logging.impl.Log4JLogger
十一月 03, 2023 5:19:24 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests 这是info日志...
十一月 03, 2023 5:19:24 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests public class java.util.logging.Logger
2023-11-03 17:19:24 DEBUG (com.feiyizhan.LogLog4jTests:21) - LogLog4jTests 这是debug日志...
2023-11-03 17:19:24  INFO (com.feiyizhan.LogLog4jTests:23) - LogLog4jTests 这是info日志...
2023-11-03 17:19:24  WARN (com.feiyizhan.LogLog4jTests:24) - LogLog4jTests 这是warn日志...
2023-11-03 17:19:24 ERROR (com.feiyizhan.LogLog4jTests:25) - LogLog4jTests 这是error日志...
2023-11-03 17:19:24  INFO (com.feiyizhan.LogLog4jTests:26) - LogLog4jTests public class org.apache.log4j.Logger
[2023-11-03 17:19:24.709] [] [main] INFO : [com.feiyizhan.LogBackTests:22] - LogBackTests 这是info日志...
[2023-11-03 17:19:24.712] [] [main] WARN : [com.feiyizhan.LogBackTests:23] - LogBackTests 这是warn日志...
[2023-11-03 17:19:24.712] [] [main] ERROR: [com.feiyizhan.LogBackTests:24] - LogBackTests 这是error日志...
[2023-11-03 17:19:24.712] [] [main] INFO : [com.feiyizhan.LogBackTests:25] - LogBackTests 这是info日志...test
[2023-11-03 17:19:24.714] [] [main] INFO : [com.feiyizhan.LogBackTests:26] - LogBackTests public final class ch.qos.logback.classic.Logger
2023-11-03 17:19:25.185  INFO 11800 --- [           main] c.f.LogLog4j2Tests                       : LogLog4j2Tests 这是info日志...
2023-11-03 17:19:25.189  WARN 11800 --- [           main] c.f.LogLog4j2Tests                       : LogLog4j2Tests 这是warn日志...
2023-11-03 17:19:25.189 ERROR 11800 --- [           main] c.f.LogLog4j2Tests                       : LogLog4j2Tests 这是error日志...
2023-11-03 17:19:25.194  INFO 11800 --- [           main] c.f.LogLog4j2Tests                       : LogLog4j2Tests 这是info日志...test
2023-11-03 17:19:25.194  INFO 11800 --- [           main] c.f.LogLog4j2Tests                       : LogLog4j2Tests public class org.apache.logging.log4j.core.Logger
```

## 引入对应over依赖包的情况下的输出
全部统一一种日志输出格式了。
```log
[2023-11-03 17:23:25.929] [] [main] INFO : [com.feiyizhan.LogJclTests:23] - LogJclTests 这是info日志...
[2023-11-03 17:23:25.929] [] [main] WARN : [com.feiyizhan.LogJclTests:24] - LogJclTests 这是warn日志...
[2023-11-03 17:23:25.929] [] [main] ERROR: [com.feiyizhan.LogJclTests:25] - LogJclTests 这是error日志...
[2023-11-03 17:23:25.930] [] [main] INFO : [com.feiyizhan.LogJclTests:26] - LogJclTests public class org.apache.commons.logging.impl.SLF4JLocationAwareLog
[2023-11-03 17:23:25.931] [] [main] INFO : [com.feiyizhan.LogJulTests:19] - LogJulTests 这是info日志...
[2023-11-03 17:23:25.931] [] [main] INFO : [com.feiyizhan.LogJulTests:20] - LogJulTests public class java.util.logging.Logger
[2023-11-03 17:23:25.937] [] [main] INFO : [com.feiyizhan.LogLog4jTests:23] - LogLog4jTests 这是info日志...
[2023-11-03 17:23:25.937] [] [main] WARN : [com.feiyizhan.LogLog4jTests:24] - LogLog4jTests 这是warn日志...
[2023-11-03 17:23:25.939] [] [main] ERROR: [com.feiyizhan.LogLog4jTests:25] - LogLog4jTests 这是error日志...
[2023-11-03 17:23:25.939] [] [main] INFO : [com.feiyizhan.LogLog4jTests:26] - LogLog4jTests public class org.apache.log4j.Logger
[2023-11-03 17:23:25.940] [] [main] INFO : [com.feiyizhan.LogBackTests:22] - LogBackTests 这是info日志...
[2023-11-03 17:23:25.943] [] [main] WARN : [com.feiyizhan.LogBackTests:23] - LogBackTests 这是warn日志...
[2023-11-03 17:23:25.943] [] [main] ERROR: [com.feiyizhan.LogBackTests:24] - LogBackTests 这是error日志...
[2023-11-03 17:23:25.946] [] [main] INFO : [com.feiyizhan.LogBackTests:25] - LogBackTests 这是info日志...test
[2023-11-03 17:23:25.949] [] [main] INFO : [com.feiyizhan.LogBackTests:26] - LogBackTests public final class ch.qos.logback.classic.Logger
[2023-11-03 17:23:26.071] [] [main] INFO : [com.feiyizhan.LogLog4j2Tests:20] - LogLog4j2Tests 这是info日志...
[2023-11-03 17:23:26.071] [] [main] WARN : [com.feiyizhan.LogLog4j2Tests:21] - LogLog4j2Tests 这是warn日志...
[2023-11-03 17:23:26.071] [] [main] ERROR: [com.feiyizhan.LogLog4j2Tests:22] - LogLog4j2Tests 这是error日志...
[2023-11-03 17:23:26.083] [] [main] INFO : [com.feiyizhan.LogLog4j2Tests:23] - LogLog4j2Tests 这是info日志...test
[2023-11-03 17:23:26.084] [] [main] INFO : [com.feiyizhan.LogLog4j2Tests:24] - LogLog4j2Tests public class org.apache.logging.slf4j.SLF4JLogger
```
