# Java Log SLF4j simple binding demo
测试slf4j binding （桥接到 `Slf4j-simple` ）的场
将原本的`Logback`、`Log4j2`、`Jul`、`Jcl`、`log4j`的输出，统一到`Sfl4j` 并最终使用`slf4j-simple`输出日志
要实现这个效果就必须引入对应日志框架的`Slf4j`的Over包，将日志重定向到`Slf4j`。但由于`Slf4j` 并没有一个具体的日志实现，
所以还得找一个具体的日志实现，这个demo选择的是`slf4j-simple`作为最终的日志实现。

## 引入对应Over包
```xml
<!-- jcl over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jcl-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- lo4j over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>log4j-over-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- jul over slf4j -->
<dependency>
    <groupId>org.slf4j</groupId>
    <artifactId>jul-to-slf4j</artifactId>
    <version>${slf4j.version}</version>
</dependency>

<!-- LOG4J2 over SLF4j -->
<dependency>
    <groupId>org.apache.logging.log4j</groupId>
    <artifactId>log4j-to-slf4j</artifactId>
    <version>${log4j2.version}</version>
</dependency>
```

## `Logback` Over 处理
`Logback` 本身就支持并使用了`slf4j`门面 ,没有要`Over`也不需要`Over`，因此不参与本测试。

## `Jcl` Over 处理
`Jcl` 由于Spring 自己做了一个Over的包`spring-jcl`，因此如果需要测试验证，还要把这个包给排除掉。
然后引入`jcl-over-slf4j` 包就可以，当然也可以用`spring-jcl`，效果一样。

## `Log4j` Over 处理
`Log4j` 除了需要引入`log4j-over-slf4j` 包外，还要排除掉`log4j` 本身的依赖包，否则Over是无效的。
```xml
<!--引入Log4j-->
<!--		<dependency>-->
<!--			<groupId>log4j</groupId>-->
<!--			<artifactId>log4j</artifactId>-->
<!--			<version>1.2.17</version>-->
<!--		</dependency>-->
```

## `Log4j2` Over 处理
`log4j2` 引入`log4j-to-slf4j` 这个包。

还需要排除`log4j-core` 包，当然如果同时存在`logback-classic` 则不排除也可以正常运行，否则会抛出异常`org.apache.logging.slf4j.SLF4JLoggerContext cannot be cast to org.apache.logging.log4j.core.LoggerContext`
。因为Spring 默认会优先加载Logback的`LoggingSystem`的一个实现`LogbackLoggingSystem`，当没有的时候才会选择`Log4J2LoggingSystem`，但当没有`Logback`实现,并且存在`log4j-core`时，
将会使用`Log4J2LoggingSystem`作为日志的实现，其实也就是一个动态的加载日志实现系统的处理。但`log4j-core`不能直接转换到`Slf4j`,因此才会抛出上面的异常。

```xml
<!--		<dependency>-->
<!--			<groupId>org.apache.logging.log4j</groupId>-->
<!--			<artifactId>log4j-core</artifactId>-->
<!--			<version>${log4j2.version}</version>-->
<!--		</dependency>-->
```

## `Jul` Over 处理
引入`jul-to-slf4j` 包。

Jul  在非Spring Boot环境下默认日志配置文件: `JVM/lib/logging.properties`
Spring Boot中 Jul 的默认配置文件 `spring-boot-2.7.9.jar!\org\springframework\boot\logging\java\logging.properties`
可以通过 在`application.properties` 增加下面的配置来指定自定义的配置文件
`logging.config=classpath:logging.properties`

参考https://www.slf4j.org/legacy.html，https://www.slf4j.org/api/org/slf4j/bridge/SLF4JBridgeHandler.html  文档，修改默认Handlers，否则Over是不生效的。
简单的方法就是在`logging.properties` 替换`handlers =java.util.logging.ConsoleHandler`配置为`handlers = org.slf4j.bridge.SLF4JBridgeHandler`


> 注意`jul-to-slf4j` 和`slf4j-jdk14` 不能同时引入，因为一个是将`Jul`重定向到`slf4j`，另一个是将`slf4j`的日志输出使用`Jul`来输出。


## `SLF4J` Binding `Slf4j-Simple` 的处理

引入`slf4j` 的`Simple`实现包`slf4j-simple`
```xml
<!--添加Slfj Simple依赖-->
<dependency>
  <groupId>org.slf4j</groupId>
  <artifactId>slf4j-simple</artifactId>
  <version>${slf4j.version}</version>
</dependency>
```

Spring Boot环境下，简单配置`slf4j-simple` 方式就是在`application.properties` 增加`logging.` 前缀的配置
例如
`logging.level.root = info`
更详细的配置可以增加`simplelogger.properties` 配置文件来配置
默认的日志输出格式相当于`log4j`的 `"%r [%t] %level %logger - %m%n"`


## 没有引入对应over依赖包的情况下的输出
日志各自使用各自的输出。 Spring Boot中当运行环境有`Log4j`时，`Jcl`的默认使用`Log4j`的实现。
```log
2023-11-03 18:38:01 DEBUG (com.feiyizhan.LogJclTests:21) - LogJclTests 这是debug日志...
2023-11-03 18:38:01  INFO (com.feiyizhan.LogJclTests:23) - LogJclTests 这是info日志...
2023-11-03 18:38:01  WARN (com.feiyizhan.LogJclTests:24) - LogJclTests 这是warn日志...
2023-11-03 18:38:01 ERROR (com.feiyizhan.LogJclTests:25) - LogJclTests 这是error日志...
2023-11-03 18:38:01  INFO (com.feiyizhan.LogJclTests:26) - LogJclTests public class org.apache.commons.logging.impl.Log4JLogger
十一月 03, 2023 6:38:01 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests 这是info日志...
十一月 03, 2023 6:38:01 下午 com.feiyizhan.LogJulTests printLog
信息: LogJulTests public class java.util.logging.Logger
2023-11-03 18:38:01 DEBUG (com.feiyizhan.LogLog4jTests:21) - LogLog4jTests 这是debug日志...
2023-11-03 18:38:01  INFO (com.feiyizhan.LogLog4jTests:23) - LogLog4jTests 这是info日志...
2023-11-03 18:38:01  WARN (com.feiyizhan.LogLog4jTests:24) - LogLog4jTests 这是warn日志...
2023-11-03 18:38:01 ERROR (com.feiyizhan.LogLog4jTests:25) - LogLog4jTests 这是error日志...
2023-11-03 18:38:01  INFO (com.feiyizhan.LogLog4jTests:26) - LogLog4jTests public class org.apache.log4j.Logger
2023-11-03 18:38:02.008 [main] DEBUG LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是debug日志...
2023-11-03 18:38:02.010 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是info日志...
2023-11-03 18:38:02.010 [main] WARN LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是warn日志...
2023-11-03 18:38:02.010 [main] ERROR LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是error日志...
2023-11-03 18:38:02.010 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是info日志...test
2023-11-03 18:38:02.010 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests public class org.slf4j.simple.SimpleLogger
18:38:02.022 [main] ERROR com.feiyizhan.LogLog4j2Tests - LogLog4j2Tests 这是error日志...

```

## 引入对应over依赖包的情况下的输出
全部统一一种日志输出格式了。
```log
2023-11-03 18:39:39.271 [main] DEBUG LogJclTests - LogJclTests 这是debug日志...
2023-11-03 18:39:39.271 [main] INFO LogJclTests - LogJclTests 这是info日志...
2023-11-03 18:39:39.271 [main] WARN LogJclTests - LogJclTests 这是warn日志...
2023-11-03 18:39:39.271 [main] ERROR LogJclTests - LogJclTests 这是error日志...
2023-11-03 18:39:39.271 [main] INFO LogJclTests - LogJclTests public class org.apache.commons.logging.impl.SLF4JLog
2023-11-03 18:39:39.274 [main] INFO LogJulTests - LogJulTests 这是info日志...
2023-11-03 18:39:39.274 [main] INFO LogJulTests - LogJulTests public class java.util.logging.Logger
2023-11-03 18:39:39.277 [main] DEBUG LogLog4jTests - LogLog4jTests 这是debug日志...
2023-11-03 18:39:39.277 [main] INFO LogLog4jTests - LogLog4jTests 这是info日志...
2023-11-03 18:39:39.277 [main] WARN LogLog4jTests - LogLog4jTests 这是warn日志...
2023-11-03 18:39:39.277 [main] ERROR LogLog4jTests - LogLog4jTests 这是error日志...
2023-11-03 18:39:39.277 [main] INFO LogLog4jTests - LogLog4jTests public class org.apache.log4j.Logger
2023-11-03 18:39:39.278 [main] DEBUG LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是debug日志...
2023-11-03 18:39:39.278 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是info日志...
2023-11-03 18:39:39.278 [main] WARN LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是warn日志...
2023-11-03 18:39:39.278 [main] ERROR LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是error日志...
2023-11-03 18:39:39.278 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests 这是info日志...test
2023-11-03 18:39:39.278 [main] INFO LogSlf4jSimpleTests - LogSlf4jSimpleTests public class org.slf4j.simple.SimpleLogger
2023-11-03 18:39:39.378 [main] DEBUG LogLog4j2Tests - LogLog4j2Tests 这是debug日志...
2023-11-03 18:39:39.379 [main] INFO LogLog4j2Tests - LogLog4j2Tests 这是info日志...
2023-11-03 18:39:39.379 [main] WARN LogLog4j2Tests - LogLog4j2Tests 这是warn日志...
2023-11-03 18:39:39.381 [main] ERROR LogLog4j2Tests - LogLog4j2Tests 这是error日志...
2023-11-03 18:39:39.389 [main] INFO LogLog4j2Tests - LogLog4j2Tests 这是info日志...test
2023-11-03 18:39:39.390 [main] INFO LogLog4j2Tests - LogLog4j2Tests public class org.apache.logging.slf4j.SLF4JLogger
```
